import random
import argparse
import hashlib

_128_BITS = "0xFFFF_FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_FFFF"

def parse_int(n: str) -> int:
    if n.startswith("0b") or n.startswith("0B"):
        return int(n[2:], 2)
    elif n.startswith("0o") or n.startswith("0O"):
        return int(n[2:], 8)
    elif n.startswith("0x") or n.startswith("0X"):
        return int(n[2:], 16)
    return int(n)

def custom_hash(message: str, mask: int) -> str:
    hash = hashlib.md5(message.encode())

    return int.from_bytes(hash.digest(), "big") & mask

if __name__ == "__main__":
    parser = argparse.ArgumentParser("hash", description="returns the hash value of the given message")

    parser.add_argument("-m", "--mask", default=_128_BITS, help="the bit mask")
    parser.add_argument("-l", "--limit")

    args = parser.parse_args()

    mask = parse_int(args.mask)

    H1 = custom_hash("John Smith", mask)
    collisions = [H1]
    one_way_property_break_iteration = None
    collision_free_property_break_iteration = None
    for i in range(int(args.limit)):
        H2 = custom_hash(str(i), mask)
        if H2 == H1 and one_way_property_break_iteration is None:
            one_way_property_break_iteration = i
        if H2 in collisions:
            if collision_free_property_break_iteration is None:
                collision_free_property_break_iteration = i
        else:
            collisions.append(H2)

    print("One-Way Property breaks at iteration: " + str(one_way_property_break_iteration))
    print("Collision-Free Property breaks at iteration: " + str(collision_free_property_break_iteration))